"""Test Dyson Python library cloud submodule."""

AUTH_ACCOUNT = "fe011372-8949-4fe4-a815-06a9992126e0"
AUTH_PASSWORD = "jvDII/gIG8MXCaWydBalT1Uo4KxM4NOfUaXqd5mwe6OKb5cjQJUyjC1+g0bVi2nJO4TN0ZS2PawMqc6+ITB4fA=="
AUTH_INFO = {
    "Account": AUTH_ACCOUNT,
    "Password": AUTH_PASSWORD,
}
